with Ada.Text_IO, Ada.Integer_Text_IO, Ada.Unchecked_Deallocation;
with Gnat.Debug_Pools; use Gnat.Debug_Pools;
use Ada.Text_IO, Ada.Integer_Text_IO;
procedure Pool_Test is
   N1, N2, N3, N4 : Integer;
begin
   Put("How many objects to be freed properly: ");
   Get(N1);
   Put("How many objects to be referenced danglingly: ");
   Get(N2);
   Put("How many declared objects to be freed (incorrectly): ");
   Get(N3);
   Put("How many objects to be left as garbage (unfreed, unreferenced): ");
   Get(N4);
   
   declare
      type Int_Ref is access all Integer;
      P : Debug_Pool;
      for Int_Ref'Storage_Pool use P;
      procedure Info is new Print_Info(Put_Line);
      
      procedure Free is new Ada.Unchecked_Deallocation( Integer, Int_Ref );
      type Int_Ref_Array is array(Positive range <>) of Int_Ref;
      
      Properly_Freed : Int_Ref_Array (1..N1) := (others => new Integer);
      Dangling_Freed : Int_Ref_Array (1..N2) := (others => new Integer);
      Danglers       : Int_Ref_Array (1..N2) := Dangling_Freed;
      Objs           : array (1..N3) of aliased Integer;
      Freed_Decls    : Int_Ref_Array (1..N3);
      Ref            : Int_Ref;
   begin
      Info(P);
      for I in Properly_Freed'Range loop
         Free( Properly_Freed(I) );
      end loop;
      
      Info(P);
      for I in Dangling_Freed'Range loop
         Free( Dangling_Freed(I) );
      end loop;
      
      Info(P);
      for I in Freed_Decls'Range loop
         Freed_Decls(I) := Objs(I)'Access;
         Free( Freed_Decls(I) );
      end loop;
      
      Info(P);
      for I in 1..N4 loop
         Ref := new Integer;
      end loop;
      Ref := null;
      Info(P);
   end;
end Pool_Test;
   