pragma Assertion_Policy(Check);
package Int_Stack_Pkg is
   subtype Element_Type is Integer;
   type Stack(Max_Size : Natural) is limited private;

   procedure Push(S : in out Stack; E : in Element_Type)
     with Pre => Current_Size(S) < S.Max_Size,
          Post => Current_Size(S) = Current_Size(S)'Old+1 and
                  E = Top(S);

   procedure Pop(S : in out Stack; E : out Element_Type)
     with Pre => Current_Size(S) > 0,
          Post => Current_Size(S) = Current_Size(S)'Old -1;

   function Top( S : Stack ) return Element_Type
      with Pre => Current_Size(S)>0;

   function Current_Size(S : Stack) return Natural;

   Overflow, Underflow : exception;
private
   type Element_Array is array(Positive range <>) of Element_Type;
   type Stack(Max_Size : Natural) is 
      record
         Current_Size : Natural := 0;
         Data         : Element_Array(1..Max_Size);
      end record;
end Int_Stack_Pkg;


   
